'use strict';
const ManagerController = require('../manager.js');
/**
 * [exports 商品类型属性控制器]
 * @type {[type]}
 */
module.exports = class ProductTypesItemController extends ManagerController {
    /**
     * [useModel 使用模型]
     * @return {[type]} [description]
     */
    get useModel() {
        let that = this;
        return that.app.model.ProductTypesItem;
    }
    /**
     * [createValidate 添加验证器]
     * @return {[type]} [description]
     */
    get createValidate() {
        let that = this;
        return {
        	type_id:that.ctx.rules.name('类型ID').required().notEmpty().number(),
            item_name:that.ctx.rules.name('属性名称').required().notEmpty().trim(),
            item_type:that.ctx.rules.default(1).number(),
            item_value:that.ctx.rules.default('').required(),
            is_search:that.ctx.rules.default(0).number(),
            admin_id:that.ctx.rules.default(that.service.manager.ActionAdminUserId()).number(),
            create_time:that.ctx.rules.default(that.app.szjcomo.date('Y-m-d H:i:s')).required()
        };
    }
    /**
     * [selectValidate 查询验证器]
     * @return {[type]} [description]
     */
    get selectValidate() {
        let that = this;
        return {
            type_id:that.ctx.rules.name('商品类型ID').required().notEmpty(),
            page:that.ctx.rules.default(1).number(),
            limit:that.ctx.rules.default(20).number()
        };
    }
    /**
     * [selectItemsValidate 查询item列表]
     * @return {[type]} [description]
     */
    get selectItemsValidate() {
        let that = this;
        return {
            item_id:that.ctx.rules.name('属性ID').required().notEmpty(),
            page:that.ctx.rules.default(1).number(),
            limit:that.ctx.rules.default(20).number()
        };
    }

    /**
     * [selectOptions 查询条件]
     * @return {[type]} [description]
     */
    async selectOptions(data = {}) {
        let that = this;
        let options = {offset:(data.page - 1) * data.limit,limit:data.limit,include:[
            {model:that.app.model.AdminUser,as:'admin_user',attributes:[]},
            {model:that.app.model.ProductTypes,as:'product_types',attributes:[]}
        ],attributes:{
            exclude:['admin_id'],
            include:[
                [that.app.Sequelize.col('admin_user.username'),'username'],
                [that.app.Sequelize.col('product_types.type_name'),'type_name']
            ]
        },order:[['item_id','desc']],where:{type_id:{
            [that.app.Sequelize.Op.in]:data.type_id.split(',').map(item => Number(item))
        }}};
        return options;
    }
    /**
     * [selectItems 查询Item列表]
     * @return {[type]} [description]
     */
    async selectItems() {
        let that = this;
        try {
            let data = await that.ctx.validate(that.selectItemsValidate,await that.ctx.getParse());
            let options = {offset:(data.page - 1) * data.limit,limit:data.limit,where:{},attributes:['item_name','item_type','item_value','item_id']};
            options.where.item_id = {[that.app.Sequelize.Op.in]:data.item_id.split(',').map(item => Number(item))};
            let result = await that.useModel.findAll(options);
            return that.ctx.appJson(that.app.szjcomo.appResult('SUCCESS',result,false));
        } catch(err) {
            return that.ctx.appJson(that.app.szjcomo.appResult(err.message));
        }
    }

}
